local COOKIE_MAX_AGE = 3600 

local json = require("json")
local utils = require("utils")
local const = require("const")
local redis = require("resty.redis")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local ck = require("resty.cookie")
local pl_pretty = require("pl.pretty")
local memkv = ngx.shared.memkv

function hmac_sha256(key, data)
    local hmacsha256 = hmac:new(key, hmac.ALGOS.SHA256)
    if not hmac then
        return nil
    end

    local ok = hmacsha256:update(data)
    if not ok then
        return nil
    end

    local hmac_binary = hmacsha256:final()
    return hmac_binary
end


local request_method = ngx.var.request_method

if "POST" ~= request_method then
    utils.error_exit(400, "unsupported request method")
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()

user = params["UserName"]
passwd = params["Password"]

utils.check_username(user)
utils.check_passwd(passwd)

login_token = resty_string.to_hex(resty_random.bytes(32))

local cookie, err = ck:new()
if not cookie then
    utils.error_exit(500)
end

local function PKCS5Padding(data, size)
    local ret = nil
    if data == nil then
        return nil
    end

    local len = #data 
    local padding = size - len%size
    local i = padding
    if padding >= 10 then
        padding = padding + 6
    end

    while(i > 0)
    do
       data = data..string.char(padding)
        i = i - 1
    end    
    return data
end

ok, msg = utils.get_data(const.data_key.eweb.user)
if (ok) then
    if (msg) then
        user_list = json.decode(msg)
        if ((nil == user_list) or (nil == user_list["UserList"]) or (nil == user_list["UserList"][user])) then
            ngx.say([[{"code": 400, "message": "Invalid username or password", "data": {}}]])
            ngx.exit(ngx.HTTP_OK)
        end

        user_info = user_list["UserList"][user]
        salt = user_info["PasswordSalt"]
        res_hex = utils.hmac_sha256(salt, passwd)
        if (nil == res_hex) then
            utils.error_exit(500)
        end

        if (user_info["PasswordHash"] == resty_string.to_hex(hmac_sha256(salt, passwd))) then
            local permission = ""
            for _, val in ipairs(user_info["Permission"]) do
                permission = permission .. "," .. val
            end
    --if user name is admin, we should make sure the admin user have the permission to call function compute.
            if((user == "admin") and string.find(permission, "FunctionComputeManage") == nil) then
                permission = permission .. ",FunctionComputeManage"
            end

            memkv:set(login_token, permission, COOKIE_MAX_AGE)
            memkv:set(login_token .. "." .. "username", user, COOKIE_MAX_AGE)

            -- set login cookie
            local ok, err = cookie:set({
                key = "loginToken",
                value = login_token,
                path = "/",
                max_age = COOKIE_MAX_AGE,
            })
            if not ok then
                utils.error_exit(500)
            end

            user_list["UserList"][user]["LastLoginTime"] = tostring(os.time())
            user_raw = json.encode(user_list)
            ok, err_m = utils.set_data(const.data_key.eweb.user, user_raw)
            if (ok) then
                res_raw = string.format('{"UserName": "%s"}', user)
                utils.error_exit(200, res_raw)
            else
                utils.error_exit(500, "store user info failed")
            end
        else
            utils.error_exit(400)
        end

    else
        if (("admin" == user) and ("admin1234" == passwd)) then
            memkv:set(login_token, "GatewayManage", COOKIE_MAX_AGE) --have't been configured, 1 hour keepalive
            memkv:set(login_token .. "." .. "username", user, COOKIE_MAX_AGE)
            memkv:set(login_token .. "." .. user, user, COOKIE_MAX_AGE)

            -- set login cookie
            local ok, err = cookie:set({
                key = "loginToken",
                value = login_token,
                path = "/",
                max_age = COOKIE_MAX_AGE,
            })
            if not ok then
                utils.error_exit(500)
            end

            utils.error_exit(200, '{"UserName": "admin"}')
        else
            utils.error_exit(400)
        end
    end
else
    utils.log_error(msg)
end


------ EOF ------
