local json = require("json")
local utils = require("utils")
local const = require("const")
local Set = require("set")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("TSLManage")

function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

local function get_pk_from_TSL(tsl)
    local args = tsl
    if args then
        if args["profile"] then
            return args["profile"]["productKey"]
        end
    else
        return nil
    end
end

local function _check_array(tsl, name)

    if tsl[ name ] ~= nil and type(tsl[ name ]) ~= "table" then
        utils.error_exit(400, name.." must be json array")
    end

end

local function is_value_repeated(cfg, cur_index, name, value)
    for k, v in pairs(cfg) do
        if k > cur_index and v[name] == value then
            return true
        end
    end

    return false
end

local function _check_datatype(dt)

    t = dt["type"]
    if t == nil or type(t) ~= "string" or not
        (t == "int" or t == "float" or t == "date" or t == "bool" or t == "enum" or t == "struct" or t == "array" or t == "text" or t == "int32" or t == "double" or t == "text" or t == "int16" or t == "uint16" or t == "int32" or t == "uint32" or t == "int64" or t == "uint64" or t == "customized data") then
        return true, "data type is not supported"
    end

    specs = dt["specs"]
    if specs == nil or type(specs) ~= "table" then
        return true, "specs is not supported"
    end

    return false
end

local function _check_accessmode(access_mode)

    if not (access_mode == "r" or access_mode == "rw") then
        return true, "access mode must be r or rw"
    end

    return false
end

local function _check_properties(properties)

    if properties == nil then
        return true
    end

    for k, property in pairs(properties) do
        local identifier = property["identifier"]
        local accessMode = property["accessMode"]
        local dataType = property["dataType"]
        local desc = property["desc"]
        local code, msg = nil

        if identifier == nil or type(identifier) ~= "string" then
            return false, "properties/identifier is null"
        else
           code, msg = is_value_repeated(properties, k, "identifier", identifier)
           if code then
                return false, "properties/identifier is repeated"
           end
        end

        if accessMode == nil or type(accessMode) ~= "string" then
            return false, "properties/access mode is null"
        else
            code, msg = _check_accessmode(accessMode)
            if code then
                return false, "properties/access mode is not supported "..msg
            end
        end

        if dataType == nil or type(dataType) ~= "table" then
            return false, "data type is null"
        else
            code, msg =  _check_datatype(dataType)
            if code then
                return false, "properties/data type is not supported"
            end
        end

        if desc ~= nil and type(desc) ~= "string" then
            return false, "desc must be string"
        end
    end

    return true
end

local function _check_event_type(event)

    if not (event == "info" or event == "alert" or event == "error") then
        return true, "event type is not supported"
    end
    return false
end

local function _check_input_output_data(data)

    if nil == data then
        return false
    end
    for k, v in pairs(data) do
        local identifier = v["identifier"]
        local dataType = v["dataType"]
        local name = v["name"]
        local code, msg = nil
        if identifier == nil or type(identifier) ~= "string" then
                return true, "identifier is null"
        else
            code, msg = is_value_repeated(data, k, "identifier", identifier)
            if code then
                return true, "identifier is repeated"
            end
        end

        if dataType == nil or type(dataType) ~= "table" then
                return true, "data type is null"
        else
            code, msg = _check_datatype(dataType)
            if code then
                return true, "data type is not supported: "..msg
            end
        end

        if name ~= nil and type(name) ~= "string" then
            return true, "name must be string"
        end
    end
    return false
end

local function _check_events(events)

    if events == nil then
        return true
    end

    for k, event in pairs(events) do
        local identifier = event["identifier"]
        local event_type = event["type"]
        local output = event["outputData"]
        local desc = event["desc"]
        local code, msg = nil
        if identifier == nil or type(identifier) ~= "string" then
            return false, "events/identifier is null"
        else
            code, msg = is_value_repeated(events, k, "identifier", identifier)
            if code then
                return false, "events/identifier is repeated"
            end
        end

        if event_type == nil or type(event_type) ~= "string" then
            return false, "event type is null "
        else
            code, msg = _check_event_type(event_type)
            if code then
                return false, "events/event_type is supported "..msg
            end
        end

        if output ~= nil and type(output) ~= "table" then
            return false, "output is null"
        else
            code, msg =  _check_input_output_data(output)
            if code then
                return false, "events/output is not supported: "..msg
            end
        end

        if desc ~= nil and type(desc) ~= "string" then
            return false, "desc must be string type"
        end
    end
    return true
end

local function _check_services(services)

    if services == nil then
        return true
    end

    for k, service in pairs(services) do
        local identifier = service["identifier"]
        local inputData = service["inputData"]
        local output = service["outputData"]
        local desc = service["desc"]
        local code, msg = nil

        if identifier == nil or type(identifier) ~= "string"
            or is_value_repeated(services, k, "identifier", identifier) then
            return false, "identifier is null or repeated"
        end

        if inputData ~= nil and type(inputData) ~= "table" then
            return false, "input is null"
        else
            code, msg =  _check_input_output_data(input)
            if code then
                return false, "services/input is  not supported: "..msg
            end
        end

        if output ~= nil and type(output) ~= "table" then
            return false, "output is null "
        else
            code, msg = _check_input_output_data(output)
            if code then
                return false, "services/output is  not supported: "..msg
            end
        end

        if desc ~= nil and type(desc) ~= "string" then
            return false, "desc must be string"
        end
    end

    return true
end

local function check_extend_tsl_valid(extendTSL, pk_real)
    if (extendTSL)  and type(extendTSL) == "table" and tablelength(extendTSL) > 0 then
        if extendTSL["profile"] == nil or type(extendTSL["profile"]) ~= "table" then
            return false, "extend tsl profile is not invalid."
        end
        local pk = extendTSL["profile"]["productKey"]
        if pk ~= pk_real then
            return false, "product key in extend tsl is not equal to standard tsl"
        end
    end
    return true
end

local function check_standard_tsl_valid(tsl)
    _check_array(tsl, "properties")
    _check_array(tsl, "events")
    _check_array(tsl, "services")

    local code, msg = _check_properties(tsl["properties"])
    if code == false then
        return false, msg
    end
    code, msg = _check_events(tsl["events"])

    if code == false then
        return false, msg
    end

    code, msg = _check_services(tsl["events"])

    if code == false then
        return false, msg
    end

    return true
end

local function get_product_name_from_TSL(tsl)
    local args = tsl
    if args then
        if args["profile"] then
            return args["profile"]["productName"]
        end
    else
        return nil
    end
end

local function set_product_name_to_TSL(tsl, product_name)
    local args = tsl
    if args["profile"] then
        args["profile"]["productName"] = product_name
    end
    return args
end


if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local standardTSL = params["standardTSL"]
    local extendTSL = params["extendTSL"]
    local productName = params["productName"]

    local productKey = get_pk_from_TSL(standardTSL)

    if (productKey == nil) then
        productKey = get_pk_from_TSL(extendTSL)
    end

    utils.check_pk(productKey)

    if productName == nil then
        productName = get_product_name_from_TSL(standardTSL)
    else
        standardTSL = set_product_name_to_TSL(standardTSL, productName)
    end

    local code, msg = check_standard_tsl_valid(standardTSL)
    if code == false then
        utils.error_exit(400, "standard tsl is invalid: "..msg)
    end

    if extendTSL then
        code, msg =  check_extend_tsl_valid(extendTSL, productKey)
        if code == false then
            utils.error_exit(400, "extend tsl is invalid: "..msg)
        end
    end
    local ret_code = false
    -- set standard tsl to config-manager
    if (standardTSL) then
        ret_code = utils.set_data(const.data_key.tsl.standard_tsl_prefix..productKey, json.encode(standardTSL))
    end
    -- set extend tsl to config-manager
    if (extendTSL)  and type(extendTSL) == "table" and tablelength(extendTSL) > 0 then
        ret_code = utils.set_data(const.data_key.tsl.extend_tsl_prefix..productKey, json.encode(extendTSL))
    end

    if(ret_code == true) then
        utils.error_exit(200, json.encode("succeed to import TSL"))
    else
        utils.error_exit(500, json.encode("failed to import TSL"))
    end

end
utils.error_exit(400, "unsupported request method")

------ EOF ------
